package environment

import error.ErrorManager
import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS
import org.junit.jupiter.api.Assertions.assertThrows
import java.util.HashSet
import java.util.function.Function

@TestInstance(PER_CLASS)
class RuntimeEnvironmentTest {

    companion object {
        private const val TERMINATED_SUCCESSFULLY = "TERMINATED_SUCCESSFULLY"
        private const val TERMINATED_EXCEPTIONALLY = "TERMINATED_EXCEPTIONALLY"
    }

    private var indicatorSet: MutableSet<String>? = null

    @BeforeEach
    fun setUp() {
        indicatorSet = HashSet()
        RuntimeEnvironment.reset()
    }

    @AfterEach
    fun tearDown() {
        RuntimeEnvironment.reset()
    }

    @Test
    fun assignInputName() {
        RuntimeEnvironment.inputName = "test"

        assertThat(RuntimeEnvironment.inputName).isEqualTo("test")
    }

    @Test
    fun assignInput() {
        RuntimeEnvironment.input = System.`in`

        assertThat(RuntimeEnvironment.input).isEqualTo(System.`in`)
    }

    @Test
    fun assignOutput() {
        RuntimeEnvironment.output = System.out

        assertThat(RuntimeEnvironment.output).isEqualTo(System.out)
    }

    @Test
    fun assignErrorOutput() {
        RuntimeEnvironment.errorOutput = System.err

        assertThat(RuntimeEnvironment.errorOutput).isEqualTo(System.err)
    }

    @Test
    fun assignErrorManager() {
        val errorManager = ErrorManager()
        RuntimeEnvironment.errorManager = errorManager

        assertThat(RuntimeEnvironment.errorManager).isEqualTo(errorManager)
    }

    @Test
    fun assignPath() {
        RuntimeEnvironment.path = "testpath/"

        assertThat(RuntimeEnvironment.path).isEqualTo("testpath/")
    }

    @Test
    fun assignTerminationFunction() {
        RuntimeEnvironment.terminationFunction = Runnable { indicatorSet!!.add(TERMINATED_SUCCESSFULLY) }
        RuntimeEnvironment.terminateSuccessfully()

        assertThat(indicatorSet!!.contains(TERMINATED_SUCCESSFULLY)).isTrue()
    }

    @Test
    fun assignErrorTerminationFunction() {
        RuntimeEnvironment.errorTerminationFunction = Runnable { indicatorSet!!.add(TERMINATED_EXCEPTIONALLY) }
        RuntimeEnvironment.terminateExceptionally()

        assertThat(indicatorSet!!.contains(TERMINATED_EXCEPTIONALLY)).isTrue()
    }

    @Test
    fun assignPromptDecorator() {
        RuntimeEnvironment.promptDecorator = Function { s -> "[$s]" }

        assertThat(RuntimeEnvironment.decoratePrompt("test")).isEqualTo("[test]")
    }

    @Test
    fun assignValueOutputDecorator() {
        RuntimeEnvironment.valueOutputDecorator = Function { s -> "($s)" }

        assertThat(RuntimeEnvironment.decorateValueOutput("test")).isEqualTo("(test)")
    }

    @Test
    fun assignWarningOutputDecorator() {
        RuntimeEnvironment.warningOutputDecorator = Function { s -> "|$s|" }

        assertThat(RuntimeEnvironment.decorateWarningOutput("test")).isEqualTo("|test|")
    }

    @Test
    fun assignErrorOutputDecorator() {
        RuntimeEnvironment.errorOutputDecorator = Function { s -> "{$s}" }

        assertThat(RuntimeEnvironment.decorateErrorOutput("test")).isEqualTo("{test}")
    }

    @Test
    fun assignCriticalOutputDecorator() {
        RuntimeEnvironment.criticalOutputDecorator = Function { s -> "/$s/" }

        assertThat(RuntimeEnvironment.decorateCriticalOutput("test")).isEqualTo("/test/")
    }

    @Test
    fun resetWorks() {
        RuntimeEnvironment.inputName = "test"
        RuntimeEnvironment.input = System.`in`
        RuntimeEnvironment.output = System.out
        RuntimeEnvironment.errorOutput = System.err
        RuntimeEnvironment.errorManager = ErrorManager()
        RuntimeEnvironment.path = "testpath/"
        RuntimeEnvironment.terminationFunction = Runnable { indicatorSet!!.add(TERMINATED_SUCCESSFULLY) }
        RuntimeEnvironment.errorTerminationFunction = Runnable { indicatorSet!!.add(TERMINATED_EXCEPTIONALLY) }
        RuntimeEnvironment.promptDecorator = Function { s -> "[$s]" }
        RuntimeEnvironment.valueOutputDecorator = Function { s -> "($s)" }
        RuntimeEnvironment.warningOutputDecorator = Function { s -> "|$s|" }
        RuntimeEnvironment.errorOutputDecorator = Function { s -> "{$s}" }
        RuntimeEnvironment.criticalOutputDecorator = Function { s -> "/$s/" }
        RuntimeEnvironment.reset()

        assertThat(RuntimeEnvironment.inputName).isNull()
        assertThat(RuntimeEnvironment.input).isNull()
        assertThat(RuntimeEnvironment.output).isNull()
        assertThat(RuntimeEnvironment.errorOutput).isNull()
        assertThat(RuntimeEnvironment.errorManager).isNull()
        assertThat(RuntimeEnvironment.path).isNull()

        assertThrows(NullPointerException::class.java) { RuntimeEnvironment.terminateSuccessfully() }
        assertThrows(NullPointerException::class.java) { RuntimeEnvironment.terminateExceptionally() }
        assertThrows(NullPointerException::class.java) { RuntimeEnvironment.decoratePrompt("") }
        assertThrows(NullPointerException::class.java) { RuntimeEnvironment.decorateValueOutput("") }
        assertThrows(NullPointerException::class.java) { RuntimeEnvironment.decorateWarningOutput("") }
        assertThrows(NullPointerException::class.java) { RuntimeEnvironment.decorateErrorOutput("") }
        assertThrows(NullPointerException::class.java) { RuntimeEnvironment.decorateCriticalOutput("") }
    }
}
